require_relative 'canonicalizer'
require_relative 'api_impl'
require_relative 'config'
require_relative 'index_page'
require 'jekyll'
require 'json'
require 'safe_yaml'

module TeamApi
  # Functions for generating JSON objects as part of an API
  class Api
    BASEURL = 'api'

    # Generates all of the API endpoints.
    # +site+:: Jekyll site object
    def self.generate_api(site)
      impl = ApiImpl.new site, BASEURL
      generate_collection_endpoints impl
      generate_tag_category_endpoints impl
      impl.generate_schema_endpoint team_api_schema_location
      impl.generate_snippets_endpoints
      impl.generate_error_endpoint
      IndexPage.create site, impl.index_endpoints
    end

    # Calculates the full URL prefix for every API endpoint, used to generate
    # `self:` links. It is generated by concatenating the `url:` and
    # `baseurl:` values from _config.yml, and the Api::BASEURL value, e.g.
    # localhost:4001/api, https://team-api.18f.gov/public/api.
    def self.baseurl(site)
      File.join site.config['url'], site.config['baseurl'], BASEURL
    end

    def self.add_self_links(site)
      baseurl = self.baseurl site
      Config.endpoint_config.each do |endpoint_info|
        collection_name = endpoint_info['collection']
        (site.data[collection_name] || {}).values.each do |item|
          slug = Canonicalizer.canonicalize item[endpoint_info['item_id']]
          item['self'] = File.join baseurl, collection_name, slug
        end
      end
    end

    def self.generate_collection_endpoints(impl)
      Config.endpoint_config.each do |endpoint_info|
        impl.generate_index_endpoint_for_collection endpoint_info
        impl.generate_item_endpoints endpoint_info['collection']
      end
    end

    private_class_method :generate_collection_endpoints

    def self.generate_tag_category_endpoints(impl)
      %w(Skills Interests).each do |tag_category|
        impl.generate_tag_category_endpoint tag_category
        impl.generate_item_endpoints Canonicalizer.canonicalize(tag_category)
      end
    end

    private_class_method :generate_tag_category_endpoints

    def self.team_api_schema_location
      about_yml = Gem::Specification.find_by_name 'about_yml'
      if about_yml.nil?
        nil
      else
        "#{about_yml.gem_dir}/lib/about_yml/schema.json"
      end
    end

    private_class_method :team_api_schema_location
  end
end
